/*
 * Copyright (C) 2011-2012 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <libintl.h>
#define _(String) gettext(String)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <json/json.h>

#include "cache.h"
#include "fcache.h"
#include "http.h"
#include "list.h"
#include "log.h"
#include "lp_ws.h"
#include "lp_json.h"
#include "ppastats.h"

/** Default ws.size value for the getPublishedBinaries request. */
static const int DEFAULT_WS_SIZE = 150;

static const char *QUERY_GET_DOWNLOAD_COUNT = "?ws.op=getDownloadCount";
static const char *
QUERY_GET_DAILY_DOWNLOAD_TOTALS = "?ws.op=getDailyDownloadTotals";

static json_object *get_json_object(const char *url)
{
	json_object *obj = NULL;
	char *body;

	body = get_url_content(url, 0);

	if (body) {
		obj = json_tokener_parse(body);

		free(body);

		return obj;
	}

	return NULL;
}

static char *get_bpph_list_cache_key(const char *archive_url)
{
	char *key;

	key = malloc(strlen(archive_url + 7) + strlen("/bpph") + 1);
	sprintf(key, "%s/bpph", archive_url + 7);

	return key;
}

static struct bpph **get_bpph_list_from_cache(const char *key)
{
	char *content;
	struct bpph **list;
	json_object *json;

	content = fcache_get(key);
	if (!content)
		return NULL;

	json = json_tokener_parse(content);
	if (!json)
		return NULL;

	list = json_object_to_bpph_list(json);

	json_object_put(json);
	free(content);

	return list;
}

static char *get_last_creation_date(struct bpph **list)
{
	time_t last, t;
	struct bpph **cur;

	last = 0;

	if (list)
		for (cur = list; *cur; cur++) {
			t = (*cur)->date_created;
			if (t > last)
				last = t;
		}

	if (last)
		return time_to_str(last);
	else
		return NULL;
}

/*
 * 'archive_url': LP URL of the archive.
 * 'size': size of the reply array. Between 1-300, else default value is used.
 */
static char *create_query_get_bpph(const char *archive_url,
				   const char *status,
				   int size)
{
	static const char *default_opt = "?ws.op=getPublishedBinaries&ws.size=";
	static const char *status_opt = "&status=";
	char *url;
	size_t n;

	if (size < 1 || size > 300)
		size = DEFAULT_WS_SIZE;

	n = strlen(archive_url) + strlen(default_opt) + 3 + 1;

	if (status)
		n += strlen(status_opt) + strlen(status);

	url = malloc(n);
	sprintf(url, "%s%s%d", archive_url, default_opt, size);

	if (status) {
		strcat(url, status_opt);
		strcat(url, status);
	}

	return url;
}

struct bpph **get_bpph_list(const char *archive_url,
			    const char *pkg_status,
			    int ws_size)
{
	char *url, *key, *tmp;
	struct bpph **result;
	struct json_object *o, *bpph_json, *o_next;
	char *date;
	int ok;

	url = create_query_get_bpph(archive_url, pkg_status, ws_size);

	key = get_bpph_list_cache_key(archive_url);

	result = get_bpph_list_from_cache(key);

	if (result) {
		date = get_last_creation_date(result);

		if (date) {
			printf("Update package since: %s\n", date);

			tmp = malloc(strlen(url)
				     + strlen("&created_since_date=")
				     + strlen(date)+1);
			strcpy(tmp, url);
			strcat(tmp, "&created_since_date=");
			strcat(tmp, date);

			free(url);
			url = tmp;

			free(date);
		}
	}

	ok = 1;
	while (url) {
		o = get_json_object(url);
		free(url);
		url = NULL;

		if (!o) {
			ok = 0;
			break;
		}

		result = bpph_list_append_list(result,
					       json_object_to_bpph_list(o));

		o_next = json_object_object_get(o, "next_collection_link");

		if (o_next)
			url = strdup(json_object_get_string(o_next));

		json_object_put(o);
	}

	if (ok) {
		bpph_json = bpph_list_to_json(result);
		fcache_put(key, json_object_to_json_string(bpph_json));
		json_object_put(bpph_json);
	}

	free(key);

	return result;
}

int get_download_count(const char *archive_url)
{
	int n = strlen(archive_url) + strlen(QUERY_GET_DOWNLOAD_COUNT) + 1;
	char *url = malloc(n);
	int result;
	json_object *obj;

	strcpy(url, archive_url);
	strcat(url, QUERY_GET_DOWNLOAD_COUNT);

	obj = get_json_object(url);
	free(url);

	if (!obj)
		return -1;

	result = json_object_get_int(obj);

	json_object_put(obj);

	return result;
}

const struct distro_arch_series *get_distro_arch_series(const char *url)
{
	json_object *obj;
	const struct distro_arch_series *distro;
	char *content;

	distro = cache_get(url);
	if (distro)
		return (struct distro_arch_series *)distro;

	content = get_url_content(url, 1);

	if (!content)
		return NULL;

	obj = json_tokener_parse(content);

	free(content);

	if (!obj)
		return NULL;

	distro = json_object_to_distro_arch_series(obj);

	json_object_put(obj);

	cache_put(url, distro, (void (*)(void *))&distro_arch_series_free);

	return distro;
}

const struct distro_series *get_distro_series(const char *url)
{
	json_object *obj;
	const struct distro_series *distro;
	char *content;

	distro = cache_get(url);
	if (distro)
		return (struct distro_series *)distro;

	content = get_url_content(url, 1);

	if (!content)
		return NULL;

	obj = json_tokener_parse(content);

	free(content);

	if (!obj)
		return NULL;

	distro = json_object_to_distro_series(obj);

	json_object_put(obj);

	cache_put(url, distro, (void (*)(void *))&distro_series_free);

	return distro;
}

struct daily_download_total **get_daily_download_totals(const char *binary_url)
{
	char *url;
	json_object *obj;
	struct daily_download_total **result = NULL;

	url = malloc(strlen(binary_url)+
		     strlen(QUERY_GET_DAILY_DOWNLOAD_TOTALS)+1);

	strcpy(url, binary_url);
	strcat(url, QUERY_GET_DAILY_DOWNLOAD_TOTALS);

	obj = get_json_object(url);

	if (obj) {
		result = json_object_to_daily_download_totals(obj);
		json_object_put(obj);
	}

	free(url);

	return result;
}

